from django.contrib import admin

from .models import Campaign, Subscriber


class SubscriberAdmin(admin.TabularInline):
    model = Subscriber
    extra = 3


class CampaignAdmin(admin.ModelAdmin):
    inlines = (SubscriberAdmin, )
    list_filter = ('status', )
    list_display = (
        'campaign_id',
        'name',
        'status',
        'total_users',
    )


admin.site.register(Campaign, CampaignAdmin)



from django.db import models
from django.conf import settings

from ..tasks import SubscriberGetTokenTask


class Subscriber(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
    )
    campaign = models.ForeignKey(
        'referral.Campaign',
        on_delete=models.CASCADE,
    )
    token = models.CharField(max_length=512, blank=True, null=True)

    class Meta:
        verbose_name = 'Subscriber'
        verbose_name_plural = 'Subscribers'

    def get_campaign_token(self):
        SubscriberGetTokenTask().s(**{
            'campaign_id': self.campaign.campaign_id,
            'user_pk': self.user.pk,
        }).apply_async()



from django.conf import settings

from rest_framework import viewsets, mixins, status
from rest_framework.permissions import IsAuthenticated
from rest_framework.decorators import action
from rest_framework.response import Response

from ...models import Campaign
from ..serializers import CampaignSerializer, CampaignSubscribeSerializer


class CampaignViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    viewsets.GenericViewSet,
):
    permission_classes = (IsAuthenticated, )
    lookup_field = 'campaign_id'
    lookup_url_kwarg = 'campaign_id'

    serializers = {
        'default': CampaignSerializer,
        'list': CampaignSerializer,
        'subscribe': CampaignSubscribeSerializer,
    }

    def get_serializer_class(self):
        return self.serializers.get(
            self.action,
            self.serializers['default'],
        )

    def get_queryset(self):
        queryset = Campaign.objects.filter(status=settings.REFERRAL_CAMPAIGN_STATUS_ACTIVE)

        if self.action == 'list':
            queryset = queryset.filter(users__email=self.request.user.email)

        return queryset

    @action(methods=['post'], detail=True, url_path='subscribe')
    def subscribe(self, request, campaign_id):
        instance = self.get_object()
        serializer = self.get_serializer(instance=instance, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save(user_from=request.user)
        return Response(serializer.data, status.HTTP_200_OK)



from rest_framework import serializers

from ..models import Campaign


class CampaignSerializer(serializers.ModelSerializer):
    campaignId = serializers.CharField(source='campaign_id')
    token = serializers.SerializerMethodField()

    class Meta:
        model = Campaign
        fields = ['name', 'campaignId', 'token']

    def get_token(self, obj):
        return obj.subscriber_set.get(user=self.context.get('request').user).token


class CampaignSubscribeSerializer(serializers.Serializer):
    rh = serializers.CharField(required=True)
    conversion = serializers.BooleanField(required=False)

    def update(self, instance, validated_data):
        user_from = validated_data.get('user_from')
        rh = validated_data.get('rh')
        conversion = validated_data.get('conversion', False)

        instance.subscribe(user_from, rh, conversion)

        return validated_data



from celery import current_app as app

from .campaign import CampaignSubscribeTask
from .subscriber import SubscriberGetTokenTask

app.tasks.register(CampaignSubscribeTask())
app.tasks.register(SubscriberGetTokenTask())



def normalize_email(email):
    """
    Normalize the email address by lowercasing the domain/email name part of it.
    """
    email = email or ''
    try:
        email_name, domain_part = email.strip().rsplit('@', 1)
    except ValueError:
        pass
    else:
        email = '@'.join([email_name.lower(), domain_part.lower()])
    return email



from django.db import models

from ..conf import settings
from ..tasks import CampaignSubscribeTask
from .subscriber import Subscriber


class Campaign(models.Model):
    campaign_id = models.CharField(max_length=255)
    status = models.CharField(
        max_length=1,
        default=settings.REFERRAL_CAMPAIGN_STATUS_DEFAULT,
        choices=settings.REFERRAL_CAMPAIGN_STATUS_CHOICES,
    )
    name = models.CharField(max_length=255)
    users = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        through='referral.Subscriber',
        related_name='referrals',
        verbose_name='user',
        blank=True,
    )

    def __str__(self):
        return '{} [ {} ]'.format(self.name, self.campaign_id)

    @property
    def total_users(self):
        return self.users.count()

    def add_subscriber(self, user):
        Subscriber.objects.create(
            user=user,
            campaign=self
        )

    def subscribe(self, user_from, rh, conversion):
        data = {
            'campaign_id': self.campaign_id,
            'firstName': user_from.first_name,
            'lastName': user_from.last_name,
            'email': user_from.email,
            'rh': rh,
        }

        if conversion:
            data['conversionName'] = settings.REFERRAL_CONVERSION_NAME

        CampaignSubscribeTask().s(**data).apply_async()



from django.apps import apps
from django.db import models


from .subscriber import new_subscriber


def setup_signals():
    Subscriber = apps.get_model(
        app_label='referral',
        model_name='Subscriber',
    )

    models.signals.post_save.connect(new_subscriber, sender=Subscriber)



import requests
import logging

from django.conf import settings

from celery import Task

logger = logging.getLogger('referral')


class CampaignSubscribeTask(Task):
    ignore_result = True
    name = 'CampaignSubscribeTask'

    def run(self, *args, **kwargs):
        campaign_id = kwargs.get('campaign_id')
        email = kwargs.get('email')
        rh = kwargs.get('rh')
        url = settings.REFERRAL_EARLY_PARROT_BASE_API.format(campaign_id)

        data = {
            'firstName': kwargs.get('firstName'),
            'lastName': kwargs.get('lastName'),
            'email': email,
            'rh': rh,
        }

        if kwargs.get('conversionName'):
            data['conversionName'] = kwargs.get('conversionName')

        try:
            requests.post(url, data=data)
            logger.info('CampaignSubscribeTask: {}-{}-{}'.format(campaign_id, email, rh))
        except Exception as exc:
            logger.error('CampaignSubscribeTask.Exception: {}-{}-{}'.format(campaign_id, email, rh))
            logger.error('CampaignSubscribeTask.Exception: {}'.format(exc))



# Generated by Django 2.2.5 on 2019-09-17 06:51

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Campaign',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('campaign_id', models.CharField(max_length=255)),
                ('status', models.CharField(choices=[('A', 'Active'), ('I', 'Inactive')], default='A', max_length=1)),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Subscriber',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(blank=True, max_length=512, null=True)),
                ('campaign', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='referral.Campaign')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Subscriber',
                'verbose_name_plural': 'Subscribers',
            },
        ),
        migrations.AddField(
            model_name='campaign',
            name='users',
            field=models.ManyToManyField(blank=True, related_name='referrals', through='referral.Subscriber', to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
    ]



import requests
import logging

from django.apps import apps
from django.conf import settings

from celery import Task

logger = logging.getLogger('referral')


class SubscriberGetTokenTask(Task):
    ignore_result = True
    name = 'SubscriberGetTokenTask'

    def run(self, *args, **kwargs):
        Campaign = apps.get_model(
            app_label='referral',
            model_name='Campaign',
        )
        campaign = Campaign.objects.get(campaign_id=kwargs.get('campaign_id'))
        subscriber = campaign.subscriber_set.get(user__pk=kwargs.get('user_pk'))

        url = settings.REFERRAL_EARLY_PARROT_BASE_API.format(campaign.campaign_id)

        data = {
            'firstName': subscriber.user.first_name,
            'lastName': subscriber.user.last_name,
            'email': subscriber.user.email,
        }

        response = requests.post(url, data=data)

        subscriber.token = response.json().get('user').get('confirmationToken')
        subscriber.save()

        logger.info('SubscriberGetTokenTask: {}-{}'.format(
            campaign.campaign_id,
            subscriber.user.email,
        ))



import logging

from django.contrib.auth import get_user_model
from django.http import HttpResponse

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.permissions import AllowAny

from ...models import Campaign
from ...signals_define import referral_reward_acquired

logger = logging.getLogger('referral')


class RewardAPIView(APIView):
    permission_classes = (AllowAny,)

    def post(self, request, *args, **kwargs):
        status_code = status.HTTP_404_NOT_FOUND
        webhook_data = request.data

        campaign_id = webhook_data.get('campaignId')
        email = webhook_data.get('email')

        try:
            campaign = Campaign.objects.get(campaign_id=campaign_id)
            user_from = campaign.users.get(email=email)

            referral_reward_acquired.send(
                sender=campaign.__class__,
                user_from=user_from,
                reward_data=webhook_data,
            )
            status_code = status.HTTP_200_OK
            logger.info('RewardAPIView OK: {}-{}'.format(campaign_id, email))

        except Campaign.DoesNotExist:
            logger.error('Campaign.DoesNotExist: {}-{}'.format(campaign_id, email))

        except get_user_model().DoesNotExist:
            logger.error('User.DoesNotExist: {}-{}'.format(campaign_id, email))
        except Exception as exc:
            logger.error('RewardAPIView.Exception: {}'.format(exc))

        return HttpResponse(status=status_code)



import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model

from ...helpers import normalize_email
from ...models import Campaign

logger = logging.getLogger('referral')


class Command(BaseCommand):
    help = ('Add users to a campaign')
    missing_args_message = 'You must provide a campaign ID and CSV file path.'

    def add_arguments(self, parser):
        parser.add_argument('-c', '--campaign', nargs='+', type=str)
        parser.add_argument('-f', '--file', nargs='+', type=str)

    def _read_emails_from_path(self, filepath):
        emails = []

        with open(filepath) as file:
            emails = file.readlines()

        return emails

    def handle(self, *args, **options):
        campaign_id = options.get('campaign')[0]
        filepath = options.get('file')[0]

        logger.info('Command.add_users_to_campaign: {}'.format(campaign_id))

        try:
            campaign = Campaign.objects.get(campaign_id=campaign_id)
            emails = self._read_emails_from_path(filepath)

            for email in emails:
                email = normalize_email(email)

                try:
                    user = get_user_model().objects.get(email=email)
                    campaign.add_subscriber(user)
                    logger.info('User {} added to campaign {}'.format(email, campaign_id))
                except ObjectDoesNotExist:
                    logger.error('User {} DoesNotExist'.format(email))

        except ObjectDoesNotExist:
            logger.error('Campaign {} DoesNotExist'.format(campaign_id))



from django.conf.urls import url, include

from rest_framework.routers import DefaultRouter

from .views import CampaignViewSet, RewardAPIView

app_name = 'referral'

router = DefaultRouter()

router.register(r'campaigns', CampaignViewSet, base_name='campaign')

urlpatterns = [
    url(r'^webhooks/2XliDq2pdp1szRJ0LR9s2598G/$', RewardAPIView.as_view(), name='reward-awared'),
    url(r'^', include(router.urls)),
]
